#include "stdafx.h"
#include "GameLevel.h"
#include <fstream>


GameLevel::GameLevel():
m_width(-1),
m_height(-1),
m_horizonGridCnt(-1),
m_verticalGridCnt(-1),
m_gridSize(25),
m_tileSize(m_gridSize*2),
m_strResourceDir("")
{
}

GameLevel::~GameLevel()
{
}

void GameLevel::New()
{
	m_horizonGridCnt	= m_width / m_gridSize;
	m_verticalGridCnt	= m_height / m_gridSize;
	m_tileSize			= m_gridSize * 2;

	int hTileCnt		= m_horizonGridCnt / 2;
	int vTileCnt		= m_verticalGridCnt / 2;
	//䲢ʼTileϢ
	m_vecTileInfo.resize(vTileCnt);
	for (int i=0; i<vTileCnt; ++i)
		m_vecTileInfo[i].resize(hTileCnt);

	for(int i=0; i<vTileCnt; ++i)
		for(int j=0; j<hTileCnt; ++j)
			m_vecTileInfo[i][j].Reset();
}

void GameLevel::Save( const char* filename )
{
	std::ofstream file(filename, std::ios::binary);

	//
	size_t texCnt = m_vecFileNames.size();
	file.write((char*)&texCnt, sizeof(size_t));

	for(size_t i=0; i<m_vecFileNames.size(); ++i)
	{
		int len = m_vecFileNames[i].GetLength();
		file.write((char*)&len, sizeof(int));
		//ļ
		const std::string name = m_vecFileNames[i];
		file.write(name.c_str(), len*sizeof(char));
		
		//߼
		eGridProperty prop = m_vecProp[i];
		//һҪӦһ߼,Ĭϰ
		if(prop == eGridProp_None)
			prop = eGridProp_Obstacle;
		file.write((char*)&prop, sizeof(eGridProperty));
	}

	int hTileCnt = m_horizonGridCnt / 2;
	int vTileCnt = m_verticalGridCnt / 2;
	//Tile
	file.write((char*)&hTileCnt, sizeof(int));
	file.write((char*)&vTileCnt, sizeof(int));

	for (int y=0; y<vTileCnt; ++y)
	{
		for (int x=0; x<hTileCnt; ++x)
		{
			const TileInfo& info = GetTileInfo(x, y);

			//TileӦ
			file.write((const char*)&info.textureIndex, sizeof(int));

			//Tile4
			bool bLT, bLB, bRT, bRB;
			bLT = bLB = bRT = bRB = false;
			switch (info.transform)
			{
			case GameLevel::eTT_Whole:	bLT = bLB = bRT = bRB = true; break;
			case GameLevel::eTT_Top:	bLT = bRT = true; break;
			case GameLevel::eTT_Left:	bLT = bLB = true; break;
			case GameLevel::eTT_Right:	bRT = bRB = true; break;
			case GameLevel::eTT_Bottom: bLB = bRB = true; break;
			}
			
			file.write((char*)&bLT, sizeof(bool));
			file.write((char*)&bLB, sizeof(bool));
			file.write((char*)&bRT, sizeof(bool));
			file.write((char*)&bRB, sizeof(bool));
		}
	}

	file.close();
}

void GameLevel::Open( const char* filename )
{
	std::ifstream file(filename, std::ios::binary);

	//
	size_t texCnt = 0;
	file.read((char*)&texCnt, sizeof(size_t));
	m_vecFileNames.resize(texCnt);

	for(size_t i=0; i<texCnt; ++i)
	{
		int len = 0;
		file.read((char*)&len, sizeof(int));
		//ļ
		char texName[MAX_PATH];
		file.read(texName, len*sizeof(char));
		texName[len] = 0;

		m_vecFileNames[i] = texName;

		//߼
		eGridProperty prop;
		file.read((char*)&prop, sizeof(eGridProperty));
		m_vecProp[i] = prop;
	}

	int hTileCnt = 0;
	int vTileCnt = 0;
	//Tile
	file.read((char*)&hTileCnt, sizeof(int));
	file.read((char*)&vTileCnt, sizeof(int));

	m_horizonGridCnt = hTileCnt * 2;
	m_verticalGridCnt = vTileCnt * 2;

	m_vecTileInfo.resize(vTileCnt);
	for (int i=0; i<vTileCnt; ++i)
		m_vecTileInfo[i].resize(hTileCnt);

	for (int y=0; y<vTileCnt; ++y)
	{
		for (int x=0; x<hTileCnt; ++x)
		{
			TileInfo& info = m_vecTileInfo.at(y).at(x);
			//TileӦ
			file.read((char*)&info.textureIndex, sizeof(int));

			bool bLT, bLB, bRT, bRB;
			//Tile4ӵ
			file.read((char*)&bLT, sizeof(bool));
			file.read((char*)&bLB, sizeof(bool));
			file.read((char*)&bRT, sizeof(bool));
			file.read((char*)&bRB, sizeof(bool));

			if(bLT && bLB && bRT && bRB)
				info.transform = eTT_Whole;
			else if(bLT && bLB)
				info.transform = eTT_Left;
			else if(bLT && bRT)
				info.transform = eTT_Top;
			else if(bRT && bRB)
				info.transform = eTT_Right;
			else if(bLB && bRB)
				info.transform = eTT_Bottom;
			else
				info.transform = eTT_None;
		}
	}

	file.close();
}

void GameLevel::SetResourceDirectory( const CString& dir )
{
	m_strResourceDir = dir;
	m_vecFileNames.clear();
	m_vecProp.clear();
	//Ŀ¼ļ
	CString fileToFind = dir + "*.*";
	WIN32_FIND_DATA data;
	HANDLE hFind = FindFirstFile(fileToFind, &data);
	if(hFind != INVALID_HANDLE_VALUE)
	{
		//...
		if(!(data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
			m_vecFileNames.push_back(data.cFileName);

		while(FindNextFile(hFind, &data) != 0)
		{
			if(!(data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
				m_vecFileNames.push_back(data.cFileName);
		}
	}

	m_vecProp.resize(m_vecFileNames.size());
	for(size_t i=0; i<m_vecProp.size(); ++i)
		m_vecProp[i] = GameLevel::eGridProp_None;
}

const std::wstring GameLevel::GetResFullPath( const char* filename) const
{
	CString str = m_strResourceDir + filename;

	WCHAR wstr[MAX_PATH];
	MultiByteToWideChar( CP_ACP, 0, str,
		strlen(str)+1, wstr,   
		sizeof(wstr)/sizeof(wstr[0]) );

	return wstr;
}

void GameLevel::SetTileTexture( int tileX, int tileY, int texIndex )
{
	m_vecTileInfo[tileY][tileX].textureIndex = texIndex;
	m_vecTileInfo[tileY][tileX].transform = eTT_Whole;
}

const std::wstring GameLevel::GetTileTexutreName( int tileX, int tileY ) const
{
	const TileInfo& info = m_vecTileInfo[tileY][tileX];
	if(info.textureIndex == -1)
		return L"";

	const CString& texName = m_vecFileNames[info.textureIndex];

	return GetResFullPath(texName);
}

int GameLevel::GetTileTexture( int tileX, int tileY ) const
{
	return m_vecTileInfo[tileY][tileX].textureIndex;
}

void GameLevel::SetTileTransform( int tileX, int tileY, eTileTransform tran )
{
	m_vecTileInfo[tileY][tileX].transform = tran;
}

const GameLevel::TileInfo& GameLevel::GetTileInfo( int tileX, int tileY ) const
{
	return m_vecTileInfo[tileY][tileX];
}

void GameLevel::SetTexutreProp( int texIndex, eGridProperty prop )
{
	m_vecProp[texIndex] = prop;
}

const GameLevel::eGridProperty GameLevel::GetTextureProp( int texIndex ) const
{
	return m_vecProp[texIndex];
}

