// GridEditDlg.cpp : ʵļ
//

#include "stdafx.h"
#include "LevelEditor.h"
#include "GridEditDlg.h"
#include "afxdialogex.h"
#include "GameLevel.h"
#include "View.h"



IMPLEMENT_DYNAMIC(CGridEditDlg, CDialog)

BEGIN_MESSAGE_MAP(CGridEditDlg, CDialog)
	ON_LBN_SELCHANGE(IDC_TextureList, OnTextureSelectChange)
	ON_COMMAND(IDC_RADIO1, OnPropBreakable)
	ON_COMMAND(IDC_RADIO2, OnPropUnbreakable)
	ON_COMMAND(IDC_RADIO3, OnPropSlidable)
	ON_COMMAND(IDC_RADIO4, OnPropObstacle)
	ON_COMMAND(IDC_RADIO5, OnPropInvisible)
END_MESSAGE_MAP()

CGridEditDlg::CGridEditDlg(GameLevel* pLevel, CEditorView* pView, CWnd* pParent /*=NULL*/)
	: CDialog(CGridEditDlg::IDD, pParent)
	,m_pGameLevel(pLevel)
	,m_pView(pView)
{
	
}

CGridEditDlg::~CGridEditDlg()
{
}

void CGridEditDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

void CGridEditDlg::OnGameAreaSizeChange()
{
	RECT rect;
	GetParent()->GetClientRect(&rect);
	int w = rect.right - rect.left, h = rect.bottom - rect.top;

	char szInfo[20];
	sprintf_s(szInfo, 20, "%d , %d", w, h);
	GetDlgItem(IDC_DlgSelector_AreaSize)->SetWindowText(szInfo);

	m_pGameLevel->SetGameAreaSize(w, h);
}

BOOL CGridEditDlg::OnInitDialog()
{
	BOOL ret = CDialog::OnInitDialog();

	int gridSize = m_pGameLevel->GetGridSize();
	CString strText;
	strText.Format("%d", gridSize);
	SetDlgItemText(IDC_GridSize, strText);
	UpdateData(FALSE);

	m_hList = ::GetDlgItem(GetSafeHwnd(), IDC_TextureList);

	char dir[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, dir);
	CString resPath = dir;
	resPath += "\\Media\\EditorPicture\\";
	m_pGameLevel->SetResourceDirectory(resPath);
	const std::vector<CString>& vecFileNames = m_pGameLevel->GetResourceNames();

	ListBox_ResetContent(m_hList);
	for (size_t i=0; i<vecFileNames.size(); ++i)
	{
		ListBox_AddString(m_hList, vecFileNames[i]);
	}

	return ret;
}

void CGridEditDlg::SelectNextTexture( int tileX, int tileY, bool bUpOrDown )
{
	const auto& list = m_pGameLevel->GetResourceNames();
	if(list.empty())
		return;

	int curSel = ListBox_GetCurSel(m_hList);
	int newSel;
	if (curSel == -1)
	{
		newSel = 0;
	}
	else if(curSel == 0 && bUpOrDown)
	{
		newSel = list.size() - 1;
	}
	else if (curSel == list.size() - 1 && !bUpOrDown)
	{
		newSel = 0;
	}
	else
	{
		newSel = bUpOrDown?curSel-1:curSel+1;
	}

	ListBox_SetCurSel(m_hList, newSel);

	m_pGameLevel->SetTileTexture(tileX, tileY, newSel);
}

void CGridEditDlg::OnTextureSelectChange()
{
	int curSel = ListBox_GetCurSel(m_hList);
	
	if(curSel == -1)
		return;

	const GameLevel::eGridProperty prop = m_pGameLevel->GetTextureProp(curSel);
	CheckRadioButton(IDC_RADIO1, IDC_RADIO5, IDC_RADIO1+prop);

	int tileX = m_pView->GetActiveTileX();
	int tileY = m_pView->GetActiveTileY();

	if(tileX == -1 || tileY == -1)
		return;

	m_pGameLevel->SetTileTexture(tileX, tileY, curSel);

	return;
}

void CGridEditDlg::SetSelectTexture( int index )
{
	ListBox_SetCurSel(m_hList, index);

	OnTextureSelectChange();
}

void CGridEditDlg::OnPropBreakable()
{
	int curSel = ListBox_GetCurSel(m_hList);
	if(curSel == -1)
		return;

	m_pGameLevel->SetTexutreProp(curSel, GameLevel::eGridProp_Breakable);
}

void CGridEditDlg::OnPropUnbreakable()
{
	int curSel = ListBox_GetCurSel(m_hList);
	if(curSel == -1)
		return;

	m_pGameLevel->SetTexutreProp(curSel, GameLevel::eGridProp_Unbreakable);
}

void CGridEditDlg::OnPropSlidable()
{
	int curSel = ListBox_GetCurSel(m_hList);
	if(curSel == -1)
		return;

	m_pGameLevel->SetTexutreProp(curSel, GameLevel::eGridProp_Slidable);
}

void CGridEditDlg::OnPropObstacle()
{
	int curSel = ListBox_GetCurSel(m_hList);
	if(curSel == -1)
		return;

	m_pGameLevel->SetTexutreProp(curSel, GameLevel::eGridProp_Obstacle);
}

void CGridEditDlg::OnPropInvisible()
{
	int curSel = ListBox_GetCurSel(m_hList);
	if(curSel == -1)
		return;

	m_pGameLevel->SetTexutreProp(curSel, GameLevel::eGridProp_Invisible);
}
