#include "stdafx.h"
#include "LevelEditor.h"
#include "View.h"
#include "GridEditDlg.h"
#include "GameLevel.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CEditorView

CEditorView::CEditorView():
m_bInEdit(false),
m_activeTileX(-1),
m_activeTileY(-1),
m_bFlicker(false)
{
	m_pGameLevel = new GameLevel;
	m_pGridEditDlg = new CGridEditDlg(m_pGameLevel, this);
	//ʱ.˸ǰ༭Tile
}

CEditorView::~CEditorView()
{
	delete m_pGameLevel;
	m_pGameLevel = nullptr;
	delete m_pGridEditDlg;
	m_pGridEditDlg = nullptr;
}


BEGIN_MESSAGE_MAP(CEditorView, CWnd)
	ON_WM_PAINT()
	ON_WM_CREATE()
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_KEYDOWN()
	ON_WM_TIMER()
	ON_COMMAND(IDC_FILE_NEWLEVEL, OnNewLevel)
	ON_COMMAND(IDC_FILE_OPENLEVEL, OnOpenLevel)
	ON_COMMAND(IDC_FILE_SAVELEVEL, OnSaveLevel)
END_MESSAGE_MAP()



// CEditorView message handlers

BOOL CEditorView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), reinterpret_cast<HBRUSH>(COLOR_WINDOW+1), NULL);

	return TRUE;
}

void CEditorView::OnPaint() 
{
	CPaintDC dc(this);
	
	if(!m_bInEdit)
		return;

	RECT rect;
	GetClientRect(&rect);
	int w = rect.right - rect.left;
	int h = rect.bottom - rect.top;

	Gdiplus::Graphics graphic(dc.GetSafeHdc());
	//˫
	Gdiplus::Bitmap memCanvas(w, h);
	Gdiplus::Graphics memGraphic(&memCanvas);

	//Ӹ
	int horTileCnt = m_pGameLevel->GetHorizonTileCnt();
	int verTileCnt = m_pGameLevel->GetVerticalTileCnt();
	int tileSize	= m_pGameLevel->GetTileSize();
	int gridSize	= m_pGameLevel->GetGridSize();
	Gdiplus::Pen pen(Gdiplus::Color::Brown);

	int xStart = 0;
	for (int i=0; i<horTileCnt; ++i,xStart+=tileSize)
		memGraphic.DrawLine(&pen, xStart, 0, xStart, h);

	int yStart = 0;
	for (int i=0; i<verTileCnt; ++i,yStart+=tileSize)
		memGraphic.DrawLine(&pen, 0, yStart, w, yStart);

	yStart = 0;
	//Tile
	for (int y=0; y<m_pGameLevel->GetVerticalTileCnt(); ++y,yStart+=tileSize)
	{
		xStart = 0;
		for (int x=0; x<m_pGameLevel->GetHorizonTileCnt(); ++x,xStart+=tileSize)
		{
			const GameLevel::TileInfo& info = m_pGameLevel->GetTileInfo(x, y);
			const std::wstring filename = m_pGameLevel->GetTileTexutreName(x, y);
			if(!filename.empty())
			{
				Gdiplus::Image texture(filename.c_str());
				int imageHalfW = texture.GetWidth() / 2;
				int iamgeHalfH = texture.GetHeight() / 2;
				//Tile4
				bool bLT, bLB, bRT, bRB;
				bLT = bLB = bRT = bRB = false;
				switch (info.transform)
				{
				case GameLevel::eTT_Whole:	bLT = bLB = bRT = bRB = true; break;
				case GameLevel::eTT_Top:	bLT = bRT = true; break;
				case GameLevel::eTT_Left:	bLT = bLB = true; break;
				case GameLevel::eTT_Right:	bRT = bRB = true; break;
				case GameLevel::eTT_Bottom: bLB = bRB = true; break;
				}

				if(bLT)
					memGraphic.DrawImage(&texture, Gdiplus::Rect(xStart,yStart,gridSize,gridSize)
					, 0, 0, imageHalfW, iamgeHalfH, Gdiplus::UnitPixel);
				if(bLB)
					memGraphic.DrawImage(&texture, Gdiplus::Rect(xStart,yStart+gridSize,gridSize,gridSize)
					, 0, iamgeHalfH, imageHalfW, iamgeHalfH, Gdiplus::UnitPixel);
				if(bRT)
					memGraphic.DrawImage(&texture, Gdiplus::Rect(xStart+gridSize,yStart,gridSize,gridSize)
					, imageHalfW, 0, imageHalfW, iamgeHalfH, Gdiplus::UnitPixel);
				if(bRB)
					memGraphic.DrawImage(&texture, Gdiplus::Rect(xStart+gridSize,yStart+gridSize,gridSize,gridSize)
					, imageHalfW, iamgeHalfH, imageHalfW, iamgeHalfH, Gdiplus::UnitPixel);
			}
		}
	}

	//,ԼTile˸
	if (m_bFlicker && m_activeTileX != -1 && m_activeTileY != -1)
	{
		Gdiplus::SolidBrush brush(Gdiplus::Color::Black);
		memGraphic.FillRectangle(&brush, m_activeTileX * tileSize, m_activeTileY * tileSize, tileSize, tileSize);
	}

	graphic.DrawImage(&memCanvas, 0, 0, w, h);
}

void CEditorView::OnNewLevel()
{
	//TODO:ʾ浱ǰؿ
	

	//½ؿ
	m_pGameLevel->New();

	InvalidateRect(nullptr);

	m_activeTileX = -1;
	m_activeTileY = -1;
	m_bInEdit = true;
}

void CEditorView::OnOpenLevel()
{
	CString strFilename;
	CFileDialog dlgFile(TRUE);	
	dlgFile.GetOFN().lpstrFilter = "*.level";
	dlgFile.GetOFN().lpstrDefExt = "level";
	dlgFile.GetOFN().lpstrInitialDir = (LPSTR)NULL; 
	dlgFile.GetOFN().lpstrTitle = "عؿ"; 

	if(IDOK == dlgFile.DoModal())
		strFilename = dlgFile.GetOFN().lpstrFile;

	m_pGameLevel->Open(strFilename);

	HWND hTexList = ::GetDlgItem(m_pGridEditDlg->GetSafeHwnd(), IDC_TextureList);
	const std::vector<CString>& vecFileNames = m_pGameLevel->GetResourceNames();
	ListBox_ResetContent(hTexList);
	for (size_t i=0; i<vecFileNames.size(); ++i)
	{
		ListBox_AddString(hTexList, vecFileNames[i]);
	}

	m_bInEdit = true;
	m_activeTileX = -1;
	m_activeTileY = -1;

	InvalidateRect(nullptr);
}

void CEditorView::OnSaveLevel()
{
	CString strPath;
	CFileDialog dlgFile(FALSE);	

	dlgFile.GetOFN().lpstrFilter = "*.level";
	dlgFile.GetOFN().lpstrDefExt = "level";
	dlgFile.GetOFN().lpstrInitialDir = (LPSTR)NULL; 
	dlgFile.GetOFN().Flags = dlgFile.GetOFN().Flags | OFN_OVERWRITEPROMPT; 
	dlgFile.GetOFN().lpstrTitle = "ؿ"; 

	if(IDOK == dlgFile.DoModal())
		strPath = dlgFile.GetOFN().lpstrFile;

	m_pGameLevel->Save(strPath);
}

int CEditorView::OnCreate( LPCREATESTRUCT lpCreateStruct )
{
	int ret = CWnd::OnCreate(lpCreateStruct);

	m_pGridEditDlg->Create(IDD_DialogSelector);
	m_pGridEditDlg->ShowWindow(SW_SHOW);
	m_pGridEditDlg->UpdateWindow();

	SetTimer(0, 500, nullptr);

	return ret;
}

void CEditorView::OnLButtonDown( UINT nFlags, CPoint point )
{
	//ѡĸTile
	int tileSize = m_pGameLevel->GetTileSize();
	m_activeTileX = point.x / tileSize;
	m_activeTileY = point.y / tileSize;

	int texIndex = m_pGameLevel->GetTileTexture(m_activeTileX, m_activeTileY);
	if(texIndex == -1)
		m_pGridEditDlg->OnTextureSelectChange();
	else
		m_pGridEditDlg->SetSelectTexture(texIndex);

	InvalidateRect(nullptr);
}

void CEditorView::OnRButtonDown( UINT nFlags, CPoint point )
{
	if(m_activeTileX == -1 || m_activeTileY == -1)
		return;

	const GameLevel::TileInfo& info = m_pGameLevel->GetTileInfo(m_activeTileX, m_activeTileY);
	GameLevel::eTileTransform newTran = (GameLevel::eTileTransform)((int)info.transform + 1);
	if(newTran > GameLevel::eTT_None)
		newTran = GameLevel::eTT_Whole;
	m_pGameLevel->SetTileTransform(m_activeTileX, m_activeTileY, newTran);

	InvalidateRect(nullptr);
}

void CEditorView::OnKeyDown( UINT nChar, UINT nRepCnt, UINT nFlags )
{
	if (nChar != VK_UP && nChar != VK_DOWN)
		return;

	if(m_activeTileX == -1 || m_activeTileY == -1)
		return;

	m_pGridEditDlg->SelectNextTexture(m_activeTileX, m_activeTileY, nChar == VK_UP);
}

void CEditorView::OnTimer( UINT_PTR nIDEvent )
{
	if(nIDEvent != 0)
		return;

	if(m_activeTileX == -1 || m_activeTileY == -1)
		return;

	//ֻػ漤Tile
	RECT rect;
	int tileSize = m_pGameLevel->GetTileSize();
	rect.left	= tileSize * m_activeTileX;
	rect.top	= tileSize * m_activeTileY;
	rect.right	= rect.left + tileSize;
	rect.bottom	= rect.top	+ tileSize;
	InvalidateRect(&rect);

	m_bFlicker = !m_bFlicker;
}

