#include "stdafx.h"
#include "MenuState.h"
#include "GUI/MenuItem.h"
#include "PlayingState.h"
#include "World.h"
#include <d3d9.h>



std::string CMenuState::StateName = "MenuState";

CMenuState::CMenuState():
m_pGUI(nullptr),
m_hTexCursor(0),
m_pFont(nullptr),
m_pSprite(nullptr),
m_bQuit(false)
{
	memset(m_menuItems, 0, sizeof(hgeGUIMenuItem*) * 5);
	m_pHGE = hgeCreate(HGE_VERSION);
}

CMenuState::~CMenuState()
{	
	m_pHGE->Release();
}

void CMenuState::enter()
{
	createScene();
}

void CMenuState::createScene()
{
	//ͼƬ
	m_Bkgroud.tex = m_pHGE->Texture_Load("Media\\bg.png");

	float screenW = (float)m_pHGE->System_GetState(HGE_SCREENWIDTH);
	float screenH = (float)m_pHGE->System_GetState(HGE_SCREENHEIGHT);

	for(int i=0;i<4;i++)
	{
		// Set up z-coordinate of vertices
		m_Bkgroud.v[i].z=0.5f;
		// Set up color. The format of DWORD col is 0xAARRGGBB
		m_Bkgroud.v[i].col=0xFFFFFFFF;
	}
	m_Bkgroud.v[0].x=0; m_Bkgroud.v[0].y=0; 
	m_Bkgroud.v[1].x=screenW; m_Bkgroud.v[1].y=0; 
	m_Bkgroud.v[2].x=screenW; m_Bkgroud.v[2].y=screenH; 
	m_Bkgroud.v[3].x=0; m_Bkgroud.v[3].y=screenH; 

	m_Bkgroud.blend=BLEND_ALPHABLEND | BLEND_COLORMUL | BLEND_NOZWRITE;

	//
	m_hTexCursor = m_pHGE->Texture_Load("Media\\cursor.png");

	//GUI
	m_pFont = new hgeFont("Media\\font1.fnt");
	m_pSprite = new hgeSprite(m_hTexCursor,0,0,32,32);

	m_pGUI=new hgeGUI();

	float y = screenH / 3;
	m_menuItems[0] = new hgeGUIMenuItem(1,m_pFont,0,screenW/2,y,0.0f,"Play");
	m_menuItems[1] = new hgeGUIMenuItem(2,m_pFont,0,screenW/2,y+40,0.1f,"Options");
	m_menuItems[2] = new hgeGUIMenuItem(3,m_pFont,0,screenW/2,y+80,0.2f,"Instructions");
	m_menuItems[3] = new hgeGUIMenuItem(4,m_pFont,0,screenW/2,y+120,0.3f,"Credits");
	m_menuItems[4] = new hgeGUIMenuItem(5,m_pFont,0,screenW/2,y+160,0.4f,"Exit");

	m_pGUI->AddCtrl(m_menuItems[0]);
	m_pGUI->AddCtrl(m_menuItems[1]);
	m_pGUI->AddCtrl(m_menuItems[2]);
	m_pGUI->AddCtrl(m_menuItems[3]);
	m_pGUI->AddCtrl(m_menuItems[4]);

	m_pGUI->SetNavMode(HGEGUI_UPDOWN | HGEGUI_CYCLED);
	m_pGUI->SetCursor(m_pSprite);
	m_pGUI->SetFocus(1);
	m_pGUI->Enter();
}

void CMenuState::exit()
{
	m_pGUI->Leave();

	m_pHGE->Texture_Free(m_hTexCursor);
	m_pHGE->Texture_Free(m_Bkgroud.tex);

	SAFE_DELETE(m_pFont);
	SAFE_DELETE(m_pSprite);
	SAFE_DELETE(m_pGUI);
}

void CMenuState::update(float timeSinceLastFrame)
{
	static float t=0.0f;
	float tx,ty;
	int id;
	static int lastid=0;

	// If ESCAPE was pressed, tell the GUI to finish
	if(m_pHGE->Input_GetKeyState(HGEK_ESCAPE)) { lastid=5; m_pGUI->Leave(); }

	// We update the GUI and take an action if
	// one of the menu items was selected
	id=m_pGUI->Update(timeSinceLastFrame);
	if(id == -1)
	{
		switch(lastid)
		{
		case 1:	//Playing״̬
			{
				World::GetSingleton().GetLevelManager().Reset();
				changeAppState(findByName(CPlayingState::StateName));
				
			}
			break;
		case 2:
		case 3:
		case 4:
			m_pGUI->SetFocus(1);
			m_pGUI->Enter();
			break;

		case 5: 
			shutdown();
			return;
		}
	}
	else if(id) { lastid=id; m_pGUI->Leave(); }

	// Here we update our background animation
	t+=timeSinceLastFrame;
	tx=50*cosf(t/60);
	ty=50*sinf(t/60);

	m_Bkgroud.v[0].tx=tx;        m_Bkgroud.v[0].ty=ty;
	m_Bkgroud.v[1].tx=tx+800/64; m_Bkgroud.v[1].ty=ty;
	m_Bkgroud.v[2].tx=tx+800/64; m_Bkgroud.v[2].ty=ty+600/64;
	m_Bkgroud.v[3].tx=tx;        m_Bkgroud.v[3].ty=ty+600/64;
}

void CMenuState::render()
{
	m_pHGE->Gfx_RenderQuad(&m_Bkgroud);
	m_pGUI->Render();
}
