#include "stdafx.h"
#include "Movie.h"
#include "GameDefine.h"



HgeMovieManager::HgeMovieManager()
{
	m_pHGE = hgeCreate(HGE_VERSION);
}

HgeMovieManager::~HgeMovieManager()
{
	Shutdown();
	SAFE_RELEASE(m_pHGE);
}

void HgeMovieManager::InitMovieTemplate( const std::string& resPath )
{
	//
	std::vector<HTEXTURE> vecTex;
	std::string texBorn1 = resPath + "born1.png";
	std::string texBorn2 = resPath + "born2.png";
	std::string texBorn3 = resPath + "born3.png";
	std::string texBorn4 = resPath + "born4.png";

	vecTex.push_back(m_pHGE->Texture_Load(texBorn1.c_str()));
	vecTex.push_back(m_pHGE->Texture_Load(texBorn2.c_str()));
	vecTex.push_back(m_pHGE->Texture_Load(texBorn3.c_str()));
	vecTex.push_back(m_pHGE->Texture_Load(texBorn4.c_str()));

	m_vecTex.insert(m_vecTex.end(), vecTex.begin(), vecTex.end());

	HgeMovie* pMovBorn = new HgeMovie(m_pHGE);
	for (size_t i=0; i<vecTex.size(); ++i)
	{
		HgeFrame* pFrame = new HgeFrame(m_pHGE, vecTex[i]);
		pMovBorn->AddFrame(pFrame);
	}

	m_movieTemplates.insert(std::make_pair("Born", pMovBorn));

	//ж:еο;̹
	std::vector<HTEXTURE> vecTexBlast;
	std::string texBlast1 = resPath + "blast1.png";
	std::string texBlast2 = resPath + "blast2.png";
	std::string texBlast3 = resPath + "blast3.png";
	std::string texBlast4 = resPath + "blast4.png";
	std::string texBlast5 = resPath + "blast5.png";
	std::string texBlast6 = resPath + "blast6.png";
	std::string texBlast7 = resPath + "blast7.png";
	std::string texBlast8 = resPath + "blast8.png";

	vecTexBlast.push_back(m_pHGE->Texture_Load(texBlast1.c_str()));
	vecTexBlast.push_back(m_pHGE->Texture_Load(texBlast2.c_str()));
	vecTexBlast.push_back(m_pHGE->Texture_Load(texBlast3.c_str()));
	vecTexBlast.push_back(m_pHGE->Texture_Load(texBlast4.c_str()));
	vecTexBlast.push_back(m_pHGE->Texture_Load(texBlast5.c_str()));

	m_vecTex.insert(m_vecTex.end(), vecTexBlast.begin(), vecTexBlast.end());

	HgeMovie* pMovTileBlast = new HgeMovie(m_pHGE);
	for (size_t i=0; i<1; ++i)
	{
		HgeFrame* pFrame = new HgeFrame(m_pHGE, vecTexBlast[i]);
		pFrame->GetSprite()->SetHotSpot(m_pHGE->Texture_GetWidth(vecTexBlast[i], true)/2.0f, 
			m_pHGE->Texture_GetHeight(vecTexBlast[i], true)/2.2f);
		pMovTileBlast->AddFrame(pFrame);
	}

	m_movieTemplates.insert(std::make_pair("TileBlast", pMovTileBlast));

	HgeMovie* pMovTankBlast = new HgeMovie(m_pHGE);
	for (size_t i=0; i<5; ++i)
	{
		HgeFrame* pFrame = new HgeFrame(m_pHGE, vecTexBlast[i]);
		pFrame->GetSprite()->SetHotSpot(m_pHGE->Texture_GetWidth(vecTexBlast[i], true)/2.0f, 
			m_pHGE->Texture_GetHeight(vecTexBlast[i], true)/2.0f);
		pMovTankBlast->AddFrame(pFrame);
	}

	m_movieTemplates.insert(std::make_pair("TankBlast", pMovTankBlast));
}

void HgeMovieManager::Shutdown()
{
	//ʵ
	KillAllInstances();

	//ģ
	for (auto iter=m_movieTemplates.begin(); iter!=m_movieTemplates.end(); ++iter)
		delete iter->second;
	m_movieTemplates.clear();

	//ͼƬ
	for(size_t i=0; i<m_vecTex.size(); ++i)
		m_pHGE->Texture_Free(m_vecTex[i]);
	m_vecTex.clear();
}

HgeMovie* HgeMovieManager::PlayMovie(const std::string& name, const hgeVector& pos)
{
	auto iter = m_movieTemplates.find(name);
	assert(iter != m_movieTemplates.end());

	HgeMovie* pMovieInstance = new HgeMovie(m_pHGE);
	(iter->second)->Clone(*pMovieInstance);

	pMovieInstance->SetPos(pos);

	m_movies.push_back(pMovieInstance);

	return pMovieInstance;
}

void HgeMovieManager::Update( float dt )
{
	for (auto iter=m_movies.begin(); iter!=m_movies.end(); /*++iter*/)
	{
		HgeMovie* pCurMovie = *iter;
		if(!pCurMovie->Update(dt))
		{
			pCurMovie->OnMovieEnd(pCurMovie);
			delete pCurMovie;
			iter = m_movies.erase(iter);
		}
		else
		{
			++iter;
		}
	}
}

void HgeMovieManager::Render()
{
	for (auto iter=m_movies.begin(); iter!=m_movies.end(); ++iter)
		(*iter)->Render();
}

void HgeMovieManager::KillAllInstances()
{
	for (auto iter=m_movies.begin(); iter!=m_movies.end(); ++iter)
		delete *iter;
	m_movies.clear();
}

void HgeMovie::AddFrame( HgeFrame* pFrame )
{
	m_frames.push_back(pFrame);
	m_movieLength += MOVIE_FRAME_TIME;
}

HgeMovie::HgeMovie( HGE* hge ):
m_pHGE(hge),
m_lifeTime(0),
m_movieLength(0),
m_bLoop(false),
m_bPingPang(false),
m_curFrameIndex(0),
m_existTime(0),
m_curFrameExistTime(0),
m_curPlayDir(true),
m_moviePos(0, 0),
m_eventListenr(nullptr),
m_fPauseTime(0)
{

}

HgeMovie::~HgeMovie()
{
	for(size_t i=0; i<m_frames.size(); ++i)
		delete m_frames[i];
	m_frames.clear();
}

void HgeMovie::Clone( HgeMovie& newMovie )
{
	for (size_t i=0; i<m_frames.size(); ++i)
	{
		HgeFrame* pFrame = new HgeFrame(m_pHGE, m_frames[i]->GetTexture());
		float hotX, hotY;
		m_frames[i]->GetSprite()->GetHotSpot(&hotX, &hotY);
		pFrame->GetSprite()->SetHotSpot(hotX, hotY);
		newMovie.AddFrame(pFrame);
	}
}

void HgeMovie::SetLoopInfo( float lifeTime, bool bPingPang )
{
	assert(lifeTime > m_movieLength);
	m_lifeTime	= lifeTime;
	m_bLoop		= true;
	m_bPingPang = bPingPang;
}

bool HgeMovie::Update( float dt )
{
	if (m_fPauseTime > 0)
	{
		m_fPauseTime -= dt;
		if(m_fPauseTime > 0)
			return true;
	}

	m_curFrameExistTime += dt;
	m_existTime			+= dt;

	//ѭģʽŽ
	if (m_bLoop && m_existTime >= m_lifeTime)
		return false;
	
	if(m_curFrameExistTime < MOVIE_FRAME_TIME)
		return true;

	if (m_bLoop)
	{
		if (m_bPingPang)
		{
			if (m_curPlayDir)
			{
				if(m_curFrameIndex == m_frames.size()-1)
				{
					m_curPlayDir = false;
					--m_curFrameIndex;
					m_curFrameExistTime = 0;
				}
				else
				{
					++m_curFrameIndex;
					m_curFrameExistTime = 0;
				}
			}
			else
			{
				if(m_curFrameIndex == 0)
				{
					m_curPlayDir = true;
					++m_curFrameIndex;
					m_curFrameExistTime = 0;
				}
				else
				{
					--m_curFrameIndex;
					m_curFrameExistTime = 0;
				}
			}
		}
		else
		{
			if (m_curFrameIndex == m_frames.size()-1)
			{
				//pingpangģʽͷѭ
				m_curFrameIndex = 0;
				m_curFrameExistTime = 0;
			}
		}
	}
	else if(m_curFrameIndex < m_frames.size()-1)
	{
		//ѭлһ֡
		++m_curFrameIndex;
		m_curFrameExistTime = 0;
	}
	else
	{
		//ѭŽ
		return false;
	}

	return true;
}

void HgeMovie::Render()
{
	if(m_fPauseTime > 0)
		return;

	m_frames[m_curFrameIndex]->Render(m_moviePos);
}

void HgeMovie::OnMovieEnd(HgeMovie* pMovie)
{
	if(m_eventListenr)
		m_eventListenr->OnMovieEnd(this);
}

HgeFrame::HgeFrame( HGE* hge, HTEXTURE tex ):
m_pHGE(hge)
{
	m_pSprite = new hgeSprite(tex, 0, 0, (float)m_pHGE->Texture_GetWidth(tex, true), (float)m_pHGE->Texture_GetHeight(tex, true));
}

HgeFrame::~HgeFrame()
{
	SAFE_DELETE(m_pSprite);
}

void HgeFrame::Render( const hgeVector& pos )
{
	m_pSprite->Render(pos.x, pos.y);
}
