#ifndef WORLD_H
#define WORLD_H


#include "Singleton.h"
#include "ObjectManager.h"
#include "LevelManager.h"
#include "Movie.h"


class Tile;
class Tank;
class Object;
class Bullet;
class hgeFont;
class Item;

typedef std::vector<Tile*> TileLocRow;
typedef std::vector<TileLocRow> VecTileLocation;
typedef std::vector<Object*> ObjectVector;

/************************************************************************/
/*							Ϸ                                */
/************************************************************************/

class World : public CSingleton<World>, public MovieEventListener
{
public:
	World();
	~World() {}

public:
	virtual void		OnMovieEnd(HgeMovie* pMovie);

public:
	LevelManager&		GetLevelManager() { return m_levelMgr; }
	ObjectManager&		GetObjectManager() { return m_objectMgr; }
	HgeMovieManager&	GetMovieManager()	{ return m_movieMgr; }
	const std::string&	GetResPath() const	{ return m_strResPath; }
	bool				IsPosValid(float x, float y);
	VecTileLocation&	GetVecTileLoc() { return m_vecTileLoc; }
	Tile*				GetTileLocation(int x, int y);
	void				SetTileLocation(int x, int y, Tile* pTile);
	void				SetPassedCurLevel(bool bPassed) { m_bPassLevel = bPassed; }
	void				SetGameOver(bool bOver) { m_bIsGameOver = bOver; }
	bool				GetGameOver() const		{ return m_bIsGameOver; }
	void				SetMainPlayer(Tank* player);
	Tank*				GetMainPlayer() const { return m_player; }
	void				OnFlagBeenHit();
	void				EatItemGrenade();
	void				EatItemStar();
	//ཻ
	bool				TestIntersect(const Object* pObj1, const Object* pObj2);
	bool				TestIntersect(const hgeRect& rect1, const hgeRect& rect2);
	//ӵе,û򷵻ؿ
	ObjectVector		GetBulletHitObject(const Bullet& bullet);
	//嵽ٶ
	void				AddToDestroyQueue(Object* pObj);
	//ʼ
	bool				InitWorld();
	//߼
	bool				UpdateWolrd();
	//Ⱦ
	void				RenderWorld();
	//뿪ؿ,
	void				ResetWorld();
	//
	void				DestroyWorld();
	//̹Ƿƶĳ
	bool				CanTankMoveTo(Tank* pTank, const hgeVector& pos);
	//1з̹.false˵ùؿͨ
	void				BornNewEnemy(const hgeVector& bornPos);
	//̹
	void				BornPlayer();
	//ȡ
	hgeVector			GetRandomBornPos(bool bNoTile);
	//ȡõǰؿеƷ,û򷵻ؿ.ؿͬʱֻܴһƷ
	Item*				GetCurrentItem();

private:
	//봦
	void				_UpdateInput();
	//ٶ
	void				_ProcessDestroyQueue();
	//ȾHUD
	void				_RenderHUD();
	//Զ,Ʒ
	void				_AutoGenObject();

	bool				__CanTankMoveInTile(const hgeRect& rectTank, int tileX, int tileY);

private:
	ObjectManager	m_objectMgr;
	LevelManager	m_levelMgr;
	HgeMovieManager	m_movieMgr;

	hgeQuad			m_quadHUD;
	hgeFont*		m_pFont;

	//ο鳡ͼ..ٲѯ
	VecTileLocation	m_vecTileLoc;

	Tank*			m_player;
	//
	ObjectVector	m_vecTobeDestroy;
	//ϷǷҪ
	bool			m_bIsGameOver;
	//Դ·
	std::string		m_strResPath;
	//ϳǷ񱻻
	bool			m_bHitFlag;
	
	bool			m_bPassLevel;
	int				m_bornEnemyCnt;
};


#endif