#include "stdafx.h"
#include "GameDefine.h"
#include "World.h"
#include "Bullet.h"
#include "Tank.h"
#include "Tile.h"





eObjectType Bullet::m_sType = eObjectType_Bullet;

Bullet::Bullet(hgeSprite* pSpr):
Object(pSpr),
m_bFromEnemy(true),
m_speed(0, 0)
{

}

Bullet::~Bullet()
{
	
}

void Bullet::Update()
{
	hgeVector newPos = GetPos() + m_speed;
	SetPos(newPos);
	const hgeRect& rect = GetBoundingBox();
	//߽
	if(	rect.x1 <= 0 || rect.x2 >= RESOLUTIONX ||
		rect.y1 <= 0 || rect.y2 >= RESOLUTIONY )
	{
		hgeVector blastPos(m_pos.x+BULLET_SIZE/2.0f, m_pos.y+BULLET_SIZE/2.0f);

		World::GetSingleton().GetMovieManager().PlayMovie("TileBlast", blastPos);
		
		ObjectManager& objMgr = World::GetSingleton().GetObjectManager();
		objMgr.m_pHGE->Effect_Play(objMgr.m_hitSound);

		World::GetSingleton().AddToDestroyQueue(this);	
		return;
	}

	//Ƿϳ
	float x = 15 * TILE_SIZE, y = (RESOLUTIONY / TILE_SIZE - 2) * TILE_SIZE;
	hgeRect rectFlag(x, y, x+50, y+50);
	if (World::GetSingleton().TestIntersect(GetBoundingBox(), rectFlag))
	{
		World::GetSingleton().OnFlagBeenHit();

		ObjectManager& objMgr = World::GetSingleton().GetObjectManager();
		objMgr.m_pHGE->Effect_Play(objMgr.m_blastSound);

		return;
	}

	//в
	ObjectVector hitObjs = World::GetSingleton().GetBulletHitObject(*this);
	if(hitObjs.empty())
		return;

	for(size_t i=0; i<hitObjs.size(); ++i)
	{
		if (hitObjs[i]->GetType() == eObjectType_Tank)
		{
			Tank* pHitTank = dynamic_cast<Tank*>(hitObjs[i]);
			assert(m_bFromEnemy != pHitTank->GetIsEnemy());
				//ӵͻ̹˼ٶ
			World::GetSingleton().AddToDestroyQueue(this);
			World::GetSingleton().AddToDestroyQueue(pHitTank);
		}
		else if (hitObjs[i]->GetType() == eObjectType_Tile)
		{
			Tile* pHitTile = dynamic_cast<Tile*>(hitObjs[i]);
			if (pHitTile->GetProp() == eTileProp_Unbreakable)
			{
				ObjectManager& objMgr = World::GetSingleton().GetObjectManager();
				objMgr.m_pHGE->Effect_Play(objMgr.m_hitSound);

				World::GetSingleton().AddToDestroyQueue(this);
			}
			else if (pHitTile->GetProp() == eTileProp_Breakable)
			{
				World::GetSingleton().AddToDestroyQueue(this);
				World::GetSingleton().AddToDestroyQueue(pHitTile);
			}
		}
	}
}

float Bullet::GetSize() const
{
	float size;
	if(m_bFromEnemy)
		size = ENEMY_BULLET_SIZE;
	else 
		size = BULLET_SIZE;

	return size;
}

eDirection Bullet::GetDirection() const
{
	eDirection dir;
	if (m_speed.x > 0)
		dir = eDirection_RIGHT;
	else if(m_speed.x < 0)
		dir = eDirection_LEFT;
	else if(m_speed.y < 0)
		dir = eDirection_UP;
	else if(m_speed.y > 0)
		dir = eDirection_DOWN;

	return dir;
}

void Bullet::OnDestroy()
{
	if(m_pOwner)
		m_pOwner->OnNotifyBulletDead();
}

float Bullet::GetObjectSize() const
{
	if(m_bFromEnemy)
		return ENEMY_BULLET_SIZE;
	else
		return BULLET_SIZE;
}
