#include "stdafx.h"
#include "LevelManager.h"
#include "Tile.h"
#include "World.h"
#include "Tank.h"
#include "Movie.h"
#include <fstream>
#include <io.h>




LevelManager::LevelManager():
m_curLevel(-1),
m_remainingLives(PLAYER_LIVES)
{

}

LevelManager::~LevelManager()
{
	for(size_t i=0; i<m_vecLevels.size(); ++i)
		m_vecLevels[i].LeaveLevel();
}

bool LevelManager::LoadAllLevels( const std::string& levelPath )
{
	m_vecLevels.clear();

	std::string strLev = levelPath + "\\1.level";
	if(_access(strLev.c_str(), 0) == -1)
		return false;

	//Ϊؿ,ֱӲ
	int iLev = 1;
	do 
	{
		m_vecLevels.push_back(Level());
		Level& level = m_vecLevels.back();
		if(!level.LoadLevel(strLev))
			return false;

		char num[10];
		::_itoa_s(++iLev, num, 10, 10);

		strLev = levelPath + "\\";
		strLev += num;
		strLev += ".level";
	} while (_access(strLev.c_str(), 0) != -1);

	return true;
}

bool LevelManager::EnterNextLevel()
{
	if(m_curLevel+1 >= (int)m_vecLevels.size() || World::GetSingleton().GetGameOver())
		return false;

	m_vecLevels[++m_curLevel].EnterLevel();

	return true;
}

void LevelManager::LeaveCurrentLevel()
{
	if(m_curLevel >=0 && m_curLevel < (int)m_vecLevels.size())
	m_vecLevels[m_curLevel].LeaveLevel();
}

void LevelManager::OnEnemyDestroy()
{
	assert(m_curLevel >= 0 && m_curLevel < (int)m_vecLevels.size());

	int remain = m_vecLevels[m_curLevel].GetRemainingEnemyCnt()-1;
	m_vecLevels[m_curLevel].SetRemainingEnemyCnt(remain);

	if(remain == 0)
	{
		if(m_curLevel+1 >= (int)m_vecLevels.size())
		{
			//ͨ..
			World::GetSingleton().SetGameOver(true);
		}
		else
		{
			World::GetSingleton().SetPassedCurLevel(true);
		}
	}
}

void LevelManager::OnPlayerBeenShot()
{
	World::GetSingleton().SetMainPlayer(NULL);

	if(m_remainingLives-- == 0)
	{
		//Ѿû..
		World::GetSingleton().SetGameOver(true);
	}
	else
	{
		int vTileCnt = RESOLUTIONY / (int)TILE_SIZE;

		HgeMovie* pBornEffect = World::GetSingleton().GetMovieManager().PlayMovie("Born", 
			hgeVector(0, (vTileCnt-2)*TILE_SIZE));
		pBornEffect->SetLoopInfo(2, true);
		pBornEffect->SetPauseTime(1);
		pBornEffect->AddListener(this);
	}
}

void LevelManager::OnMovieEnd( HgeMovie* pMovie )
{
	World::GetSingleton().BornPlayer();
}

void LevelManager::Reset()
{
	m_curLevel = -1;
	m_remainingLives = PLAYER_LIVES;
}



Level::Level():
m_remainingEnemies(ENEMY_PER_LEVEL)
{
	m_pHGE = hgeCreate(HGE_VERSION);
}

Level::~Level()
{
	SAFE_RELEASE(m_pHGE);
}

bool Level::LoadLevel( const std::string& path )
{
	std::ifstream file(path, std::ios::binary);
	if(file.fail())
		return false;

	//
	size_t texCnt = 0;
	file.read((char*)&texCnt, sizeof(size_t));

	const std::string& resPath = World::GetSingleton().GetResPath();
	m_vecTexs.resize(texCnt);
	for(size_t i=0; i<texCnt; ++i)
	{
		TextureInfoPair& info = m_vecTexs.at(i);

		int len = 0;
		file.read((char*)&len, sizeof(int));
		//ļ
		char texName[MAX_PATH];
		file.read(texName, len*sizeof(char));
		texName[len] = 0;

		const std::string filename = resPath + texName;
		info.first = m_pHGE->Texture_Load(filename.c_str());

		if(info.first == NULL)
			return false;

		//߼
		file.read((char*)&info.second, sizeof(eTileProp));
	}

	//Ӹ
	int hTileCnt, vTileCnt;
	//
	file.read((char*)&hTileCnt, sizeof(int));
	file.read((char*)&vTileCnt, sizeof(int));

	hTileCnt = RESOLUTIONX / (int)TILE_SIZE;
	vTileCnt = RESOLUTIONY / (int)TILE_SIZE;

	m_vecTileTexIdx.resize(vTileCnt);
	for(int i=0; i<vTileCnt; ++i)
		m_vecTileTexIdx[i].resize(hTileCnt);

	for (int y=0; y<vTileCnt/2; ++y)
	{
		for (int x=0; x<hTileCnt/2; ++x)
		{
			int texIdx;
			file.read((char*)&texIdx, sizeof(int));

			bool bLT, bLB, bRT, bRB;
			//4ӵ
			file.read((char*)&bLT, sizeof(bool));
			file.read((char*)&bLB, sizeof(bool));
			file.read((char*)&bRT, sizeof(bool));
			file.read((char*)&bRB, sizeof(bool));

			if(bLT)
				m_vecTileTexIdx[y*2][x*2] = texIdx;
			else
				m_vecTileTexIdx[y*2][x*2] = -1;

			if(bLB)
				m_vecTileTexIdx[y*2+1][x*2] = texIdx;
			else
				m_vecTileTexIdx[y*2+1][x*2] = -1;

			if(bRT)
				m_vecTileTexIdx[y*2][x*2+1] = texIdx;
			else
				m_vecTileTexIdx[y*2][x*2+1] = -1;

			if(bRB)
				m_vecTileTexIdx[y*2+1][x*2+1] = texIdx;
			else
				m_vecTileTexIdx[y*2+1][x*2+1] = -1;
		}	
	}
	file.close();

	//½ǵĵο,һ
	m_vecTileTexIdx[vTileCnt-2][0] = -1;
	m_vecTileTexIdx[vTileCnt-2][1] = -1;
	m_vecTileTexIdx[vTileCnt-1][0] = -1;
	m_vecTileTexIdx[vTileCnt-1][1] = -1;

	//ʼϳΧTile
	int brickTexIdx = -1;
	for (size_t i=0 ;i<m_vecTexs.size(); ++i)
	{
		if(m_vecTexs[i].second == eTileProp_Breakable)
		{
			brickTexIdx = i;
			break;
		}
	}

	m_vecTileTexIdx[vTileCnt-3][14] = brickTexIdx;
	m_vecTileTexIdx[vTileCnt-3][15] = brickTexIdx;
	m_vecTileTexIdx[vTileCnt-3][16] = brickTexIdx;
	m_vecTileTexIdx[vTileCnt-3][17] = brickTexIdx;
	m_vecTileTexIdx[vTileCnt-2][14] = brickTexIdx;
	m_vecTileTexIdx[vTileCnt-2][17] = brickTexIdx;
	m_vecTileTexIdx[vTileCnt-1][14] = brickTexIdx;
	m_vecTileTexIdx[vTileCnt-1][17] = brickTexIdx;
	m_vecTileTexIdx[vTileCnt-2][15] = -1;
	m_vecTileTexIdx[vTileCnt-2][16] = -1;
	m_vecTileTexIdx[vTileCnt-1][15] = -1;
	m_vecTileTexIdx[vTileCnt-1][16] = -1;

	return true;
}

void Level::EnterLevel()
{
	int hTileCnt = RESOLUTIONX / (int)TILE_SIZE, vTileCnt = RESOLUTIONY / (int)TILE_SIZE;
	World& world = World::GetSingleton();
	VecTileLocation& vecObjLoc = world.GetVecTileLoc();
	vecObjLoc.clear();
	vecObjLoc.resize(vTileCnt);
	for(int i=0; i<vTileCnt; ++i)
		vecObjLoc[i].resize(hTileCnt);

	for (int y=0; y<vTileCnt; ++y)
	{
		for(int x=0; x<hTileCnt; ++x)
		{
			int			texIdx	= m_vecTileTexIdx[y][x];
			
			if(texIdx == -1)
			{
				world.SetTileLocation(x, y, nullptr);
				continue;
			}

			HTEXTURE	tex		= m_vecTexs[texIdx].first;
			eTileProp	prop	= m_vecTexs[texIdx].second;
			hgeSprite* pSpr = new hgeSprite(tex, 0, 0, TILE_SIZE, TILE_SIZE);
			//θ
			Object* pObj = world.GetObjectManager().CreateObject(pSpr, eObjectType_Tile);
			Tile* pTile = dynamic_cast<Tile*>(pObj);
			pObj->SetPos(hgeVector(x*TILE_SIZE, y*TILE_SIZE));
			pTile->SetProp(prop);

			world.SetTileLocation(x, y, pTile);
		}
	}

	world.BornPlayer();
	world.SetPassedCurLevel(false);
	m_remainingEnemies = ENEMY_PER_LEVEL;
}

void Level::LeaveLevel()
{
	World::GetSingleton().ResetWorld();
}