#include "stdafx.h"
#include "ObjectManager.h"
#include "GameDefine.h"
#include "Tank.h"
#include "Tile.h"
#include "Bullet.h"
#include "World.h"
#include "Item.h"




ObjectManager::ObjectManager():
m_texTank(NULL),
m_texBullet(NULL),
m_texEnemyBullet(NULL),
m_texEnemy1(NULL),
m_texEnemy2(NULL),
m_fireSound(NULL),
m_hitSound(NULL),
m_blastSound(NULL),
m_startSound(NULL),
m_texFlag(NULL),
m_texFlagBroken(NULL),
m_gameOverSound(NULL),
m_texHUDTank(NULL),
m_pHUDTankSpr(NULL),
m_itemBornSound(NULL),
m_bombSound(NULL),
m_addLifeSound(NULL),
m_pFlag(NULL)
{
	m_pHGE = hgeCreate(HGE_VERSION);
}

ObjectManager::~ObjectManager()
{
	SAFE_RELEASE(m_pHGE);
}

Object* ObjectManager::CreateObject( hgeSprite* pSpr, eObjectType type )
{
	Object* pNewObject = nullptr;

	switch (type)
	{
	case eObjectType_Tank:			pNewObject = new Tank(pSpr); break;
	case eObjectType_Tile:			pNewObject = new Tile(pSpr); break;
	case eObjectType_Bullet:		pNewObject = new Bullet(pSpr); break;
	case eObjectType_Item:			pNewObject = new Item(pSpr); break;
	default:assert(0);
	}

	assert(pNewObject);
	if(type != eObjectType_Tank)
	{
		m_lstObjects.push_back(pNewObject);
	}
	else
	{
		//̹ҪȵοȻ(ݵ)
		//С,ײ,Ȼ
		m_lstObjects.insert(m_lstObjects.begin(), pNewObject);
	}

	return pNewObject;
}

void ObjectManager::DestroyObject( Object* pObj )
{
	assert(pObj);
	m_lstObjects.remove(pObj);
	delete pObj;
}

void ObjectManager::LoadRes()
{
	const std::string resPath = World::GetSingleton().GetResPath();
	std::string texTankFile = resPath + "Tank.png";
	m_texTank = m_pHGE->Texture_Load(texTankFile.c_str());
	assert(m_texTank);

	std::string texBulletFile = resPath + "Bullet.png";
	m_texBullet = m_pHGE->Texture_Load(texBulletFile.c_str());
	assert(m_texBullet);

	std::string texEnemyBulletFile = resPath + "EnemyBullet.png";
	m_texEnemyBullet = m_pHGE->Texture_Load(texEnemyBulletFile.c_str());
	assert(m_texEnemyBullet);

	std::string texEnemy1 = resPath + "Enemy1.png";
	m_texEnemy1 = m_pHGE->Texture_Load(texEnemy1.c_str());
	assert(m_texEnemy1);

	std::string texEnemy2 = resPath + "Enemy2.png";
	m_texEnemy2 = m_pHGE->Texture_Load(texEnemy2.c_str());
	assert(m_texEnemy2);

	std::string texFlag = resPath + "Flag.png";
	m_texFlag = m_pHGE->Texture_Load(texFlag.c_str());
	assert(m_texFlag);

	std::string texFlagBroken = resPath + "Flag_Broken.png";
	m_texFlagBroken = m_pHGE->Texture_Load(texFlagBroken.c_str());
	assert(m_texFlagBroken);

	std::string texHUDTank = resPath + "mintank.png";
	m_texHUDTank = m_pHGE->Texture_Load(texHUDTank.c_str());
	assert(m_texHUDTank);

	std::string fireSound = resPath + "fire.wav";
	m_fireSound = m_pHGE->Effect_Load(fireSound.c_str());
	assert(m_fireSound);

	std::string hitSound = resPath + "hit.wav";
	m_hitSound = m_pHGE->Effect_Load(hitSound.c_str());
	assert(m_hitSound);

	std::string blastSound = resPath + "blast.wav";
	m_blastSound = m_pHGE->Effect_Load(blastSound.c_str());
	assert(m_blastSound);

	std::string startSound = resPath + "start.wav";
	m_startSound = m_pHGE->Effect_Load(startSound.c_str());
	assert(m_startSound);

	std::string gameOverSound = resPath + "GameOver.mp3";
	m_gameOverSound = m_pHGE->Effect_Load(gameOverSound.c_str());
	assert(m_gameOverSound);

	std::string texGrenadeFile = resPath + "bomb.png";
	HTEXTURE texGrenade = m_pHGE->Texture_Load(texGrenadeFile.c_str());
	assert(texGrenade);
	m_vecItemTexs.push_back(texGrenade);

	std::string texStarFile = resPath + "star.png";
	HTEXTURE texStar = m_pHGE->Texture_Load(texStarFile.c_str());
	assert(texStar);
	m_vecItemTexs.push_back(texStar);

	std::string itemBornSound = resPath + "ItemBorn.mp3";
	m_itemBornSound = m_pHGE->Effect_Load(itemBornSound.c_str());
	assert(m_itemBornSound);

	std::string bombSound = resPath + "Bomb.mp3";
	m_bombSound = m_pHGE->Effect_Load(bombSound.c_str());
	assert(m_bombSound);

	std::string addLifeSound = resPath + "AddLife.mp3";
	m_addLifeSound = m_pHGE->Effect_Load(addLifeSound.c_str());
	assert(m_addLifeSound);

	m_pHUDTankSpr = new hgeSprite(m_texHUDTank, 0, 0, 30, 30);
	m_pFlag = new hgeSprite(m_texFlag, 0, 0, 50, 38);
}

void ObjectManager::UnloadRes()
{
	SAFE_DELETE(m_pHUDTankSpr);
	SAFE_DELETE(m_pFlag);

	m_pHGE->Texture_Free(m_texTank);
	m_pHGE->Texture_Free(m_texBullet);
	m_pHGE->Texture_Free(m_texEnemyBullet);
	m_pHGE->Texture_Free(m_texEnemy1);
	m_pHGE->Texture_Free(m_texEnemy2);
	m_pHGE->Texture_Free(m_texFlag);
	m_pHGE->Texture_Free(m_texFlagBroken);
	m_pHGE->Texture_Free(m_texHUDTank);
	m_texTank = NULL;
	m_texBullet = NULL;
	m_texEnemyBullet = NULL;
	m_texEnemy1 = NULL;
	m_texEnemy2 = NULL;
	m_texFlag = NULL;
	m_texFlagBroken = NULL;
	m_texHUDTank = NULL;

	m_pHGE->Effect_Free(m_fireSound);
	m_pHGE->Effect_Free(m_hitSound);
	m_pHGE->Effect_Free(m_blastSound);
	m_pHGE->Effect_Free(m_startSound);
	m_pHGE->Effect_Free(m_gameOverSound);
	m_pHGE->Effect_Free(m_itemBornSound);
	m_pHGE->Effect_Free(m_bombSound);
	m_pHGE->Effect_Free(m_addLifeSound);
	m_fireSound = NULL;
	m_hitSound = NULL;
	m_blastSound = NULL;
	m_startSound = NULL;
	m_gameOverSound = NULL;
	m_itemBornSound = NULL;
	m_bombSound = NULL;
	m_addLifeSound = NULL;

	for(size_t i=0; i<m_vecItemTexs.size(); ++i)
		m_pHGE->Effect_Free(m_vecItemTexs[i]);
	m_vecItemTexs.clear();
}

void ObjectManager::Render()
{
	for(auto iter=m_lstObjects.begin(); iter!=m_lstObjects.end(); ++iter)
		(*iter)->Render();
}

void ObjectManager::Update()
{
	for(auto iter=m_lstObjects.begin(); iter!=m_lstObjects.end(); ++iter)
	{
		Object* pObj = *iter;
		pObj->Update();
	}
}

void ObjectManager::DestroyAllObjects()
{
	for(auto iter=m_lstObjects.begin(); iter!= m_lstObjects.end(); ++iter)
		delete *iter;
	m_lstObjects.clear();
}
