#include "stdafx.h"
#include "Tank.h"
#include "World.h"
#include "Bullet.h"
#include "Item.h"




eObjectType Tank::m_sType = eObjectType_Tank;

Tank::Tank(hgeSprite* pSpr):
Object(pSpr),
m_direction(eDirection_DOWN),
m_bIsEnemy(true),
m_bCanShoot(true),
m_speed(0, TANK_SPEED), /*eDirection_DOWNӦ*/
m_restTime(1.5f),
m_moveTime(3),
m_curTime(0),
m_bIsMove(false),
m_shootTime(3),
m_pActiveBullet(nullptr)
{

}

Tank::~Tank()
{	
	
}

void Tank::Render()
{
	if(m_bVisible)
		m_pSprite->RenderStretch(m_pos.x, m_pos.y, m_pos.x+TANK_SIZE, m_pos.y+TANK_SIZE);
}

void Tank::SetDirection( eDirection dir )
{
	if(dir == m_direction)
		return;

	switch (dir)
	{
	case eDirection_DOWN:	m_pSprite->SetTextureRect(2, 10, 46, 60); m_speed = hgeVector(0, TANK_SPEED); break;
	case eDirection_LEFT:	m_pSprite->SetTextureRect(0, 95, 50, 45); m_speed = hgeVector(-TANK_SPEED, 0); break;
	case eDirection_RIGHT:	m_pSprite->SetTextureRect(0, 170, 50, 45); m_speed = hgeVector(TANK_SPEED, 0); break;
	case eDirection_UP:		m_pSprite->SetTextureRect(2, 240, 46, 50); m_speed = hgeVector(0, -TANK_SPEED); break;
	}

	m_direction = dir;
}

void Tank::Shoot()
{
	if (!m_bCanShoot)
		return;

	ObjectManager& objMgr = World::GetSingleton().GetObjectManager();
	hgeSprite* pSpr;
	if(m_bIsEnemy)
		pSpr = new hgeSprite(objMgr.m_texEnemyBullet, 0, 0, ENEMY_BULLET_SIZE, ENEMY_BULLET_SIZE);
	else
		pSpr = new hgeSprite(objMgr.m_texBullet, 0, 0, BULLET_SIZE, BULLET_SIZE);

	//ӵ
	Object* pObj = objMgr.CreateObject(pSpr, eObjectType_Bullet);
	m_pActiveBullet = dynamic_cast<Bullet*>(pObj);
	m_pActiveBullet->SetIsFromEnemy(m_bIsEnemy);
	m_pActiveBullet->SetOwner(this);

	hgeVector bulletSpeed;
	switch (m_direction)
	{
	case eDirection_DOWN: bulletSpeed = hgeVector(0, BULLET_SPEED); break;
	case eDirection_UP:		bulletSpeed = hgeVector(0, -BULLET_SPEED); break;
	case eDirection_LEFT: bulletSpeed = hgeVector(-BULLET_SPEED, 0); break;
	case eDirection_RIGHT: bulletSpeed = hgeVector(BULLET_SPEED, 0); break;
	}
	const float bulletSize = m_pActiveBullet->GetSize();
	m_pActiveBullet->SetPos(hgeVector(m_pos.x+TANK_SIZE/2-bulletSize/2, m_pos.y+TANK_SIZE/2-bulletSize/2));
	m_pActiveBullet->SetSpeed(bulletSpeed);

	objMgr.m_pHGE->Effect_Play(objMgr.m_fireSound);

	m_bCanShoot = false;
}

void Tank::Update()
{
	if (m_bIsEnemy)
	{
		//õƶ,ڵ
		m_curTime += TIME_PER_FRAME;
		if(m_curTime >= m_shootTime)
		{
			Shoot();
			m_shootTime = rand() % 1000 / 1000.0f * 2.0f;
		}

		if (m_bIsMove)
		{
			hgeVector newPos = m_pos + m_speed;
			if(World::GetSingleton().CanTankMoveTo(this, newPos))
				SetPos(newPos);

			if (m_curTime >= m_moveTime)
			{
				m_curTime = 0;
				m_bIsMove = false;
				m_moveTime = rand() % 1000 / 1000.0f * 3.0f;
			}
		}
		else
		{
			if (m_curTime >= m_restTime)
			{
				eDirection randDir = (eDirection)(rand() % eDirection_MaxCount);
				SetDirection(randDir);
				m_curTime = 0;
				m_bIsMove = true;
				m_restTime = rand() % 1000 / 1000.0f * 1.5f;
			}
		}
	}
	else
	{
		//ǷԵƷ
		Item* pItem = World::GetSingleton().GetCurrentItem();
		if (pItem && World::GetSingleton().TestIntersect(this, pItem))
		{
			pItem->TakeEffect();
			World::GetSingleton().AddToDestroyQueue(pItem);
		}
	}
}

void Tank::OnNotifyBulletDead()
{
	m_bCanShoot = true;
	m_pActiveBullet = nullptr;
}

void Tank::OnDestroy()
{
	hgeVector blastPos(m_pos.x+TANK_SIZE/2.0f, m_pos.y+TANK_SIZE/2.0f);
	World::GetSingleton().GetMovieManager().PlayMovie("TankBlast", blastPos);

	ObjectManager& objMgr = World::GetSingleton().GetObjectManager();
	objMgr.m_pHGE->Effect_Play(objMgr.m_blastSound);

	if(m_bIsEnemy)	//֪ͨؿµĵ
		World::GetSingleton().GetLevelManager().OnEnemyDestroy();
	else	//֪ͨؿұ
		World::GetSingleton().GetLevelManager().OnPlayerBeenShot();

	//FIX̬BUG
	if(m_pActiveBullet)
		m_pActiveBullet->SetOwner(nullptr);
}
