#include "stdafx.h"
#include "World.h"
#include "Tank.h"
#include "Object.h"
#include "Tile.h"
#include "Bullet.h"
#include "Item.h"
#include "hgefont.h"




World::World():
m_player(nullptr),
m_bIsGameOver(false),
m_bPassLevel(false),
m_pFont(NULL),
m_bornEnemyCnt(0),
m_bHitFlag(false)
{
	char dir[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, dir);
	m_strResPath = dir;
	m_strResPath += RES_PATH;
}

Tile* World::GetTileLocation( int x, int y )
{
	assert(IsPosValid((float)x, (float)y));
	return m_vecTileLoc[y][x];
}

void World::SetTileLocation( int x, int y, Tile* pTile )
{
	assert(IsPosValid((float)x, (float)y));
	m_vecTileLoc[y][x] = pTile;
}

bool World::IsPosValid( float x, float y )
{
	int hTileCnt = RESOLUTIONX / (int)TILE_SIZE, vTileCnt = RESOLUTIONY / (int)TILE_SIZE;
	const static float clientW = hTileCnt * TILE_SIZE;
	const static float clientH = vTileCnt * TILE_SIZE;
	if(x < 0 || x >= clientW || y < 0 || y>= clientH)
		return false;

	return true;
}

bool World::UpdateWolrd()
{
	if(m_bPassLevel)
		return false;

	_UpdateInput();

	//߼
	m_objectMgr.Update();

	_ProcessDestroyQueue();

	m_movieMgr.Update(TIME_PER_FRAME);

	_AutoGenObject();

	return true;
}

void World::_UpdateInput()
{
	if(!m_player)
		return;

	bool bMoved = false;
	hgeVector newPos = m_player->GetPos();
	eDirection newDir = m_player->GetDirection();
	if(GetAsyncKeyState(VK_UP) < 0)
	{
		newDir = eDirection_UP;
		newPos.y -= TANK_SPEED;
		bMoved = true;
	}
	else if(GetAsyncKeyState(VK_DOWN) < 0)
	{
		newDir = eDirection_DOWN;
		newPos.y += TANK_SPEED;
		bMoved = true;
	}
	else if(GetAsyncKeyState(VK_LEFT) < 0)
	{
		newDir = eDirection_LEFT;
		newPos.x -= TANK_SPEED;
		bMoved = true;
	}
	else if(GetAsyncKeyState(VK_RIGHT) < 0)
	{
		newDir = eDirection_RIGHT;
		newPos.x += TANK_SPEED;
		bMoved = true;
	}

	m_player->SetDirection(newDir);

	//Ƿƶ
	if(bMoved && CanTankMoveTo(m_player, newPos))
		m_player->SetPos(newPos);

	//
	if (GetAsyncKeyState(VK_SPACE) < 0)
		m_player->Shoot();
}

bool World::__CanTankMoveInTile( const hgeRect& rectTank, int tileX, int tileY )
{
	//ﳬ߽Tile,Ӧֹ̹ƶļ
	int hTileCnt = RESOLUTIONX / (int)TILE_SIZE, vTileCnt = RESOLUTIONY / (int)TILE_SIZE;
	if(tileX >= hTileCnt || tileY >= vTileCnt)
		return true;

	Tile* pTile = GetTileLocation(tileX, tileY);
	if(!pTile)
		return true;

	if(TestIntersect(pTile->GetBoundingBox(), rectTank) && pTile->GetProp() != eTileProp_Invisible)
		return false;

	return true;
}

bool World::CanTankMoveTo( Tank* pTank, const hgeVector& pos )
{
	hgeRect rctTank(pos.x, pos.y, pos.x+TANK_SIZE, pos.y+TANK_SIZE);
	//߽
	if (rctTank.x1 < 0 || rctTank.x2 > RESOLUTIONX ||
		rctTank.y1 < 0 || rctTank.y2 > RESOLUTIONY	)
		return false;

	///̹ͬʱռ9,ȥռݵ4,
	///ʣ5λҪཻ
	float tilePosX = pos.x - fmod(pos.x, TILE_SIZE);
	float tilePosY = pos.y - fmod(pos.y, TILE_SIZE);
	int tileIdxX = (int)(tilePosX / TILE_SIZE);
	int tileIdxY = (int)(tilePosY / TILE_SIZE);

	Tile* pTile1 = GetTileLocation(tileIdxX,	tileIdxY);
	Tile* pTile2 = GetTileLocation(tileIdxX+1, tileIdxY);
	Tile* pTile3 = GetTileLocation(tileIdxX,	tileIdxY+1);
	Tile* pTile4 = GetTileLocation(tileIdxX+1, tileIdxY+1);

	if(	(pTile1 && pTile1->GetProp() != eTileProp_Invisible) ||
		(pTile2 && pTile2->GetProp() != eTileProp_Invisible) ||
		(pTile3 && pTile3->GetProp() != eTileProp_Invisible) ||
		(pTile4 && pTile4->GetProp() != eTileProp_Invisible) )
		return false;

	//..
	if(	!__CanTankMoveInTile(rctTank, tileIdxX+2,	tileIdxY)	||
		!__CanTankMoveInTile(rctTank, tileIdxX+2,	tileIdxY+1) ||
		!__CanTankMoveInTile(rctTank, tileIdxX+2,	tileIdxY+2) ||
		!__CanTankMoveInTile(rctTank, tileIdxX,	tileIdxY+2) ||
		!__CanTankMoveInTile(rctTank, tileIdxX+1,	tileIdxY+2) )
		return false;

	//̹֮ײ
	const ObjectList& objList = m_objectMgr.GetObjects();
	for (auto iter=objList.begin(); iter!=objList.end(); ++iter)
	{
		Object* pObject = *iter;
		if (pObject->GetType() == eObjectType_Tank && pObject != pTank)
		{
			if(TestIntersect(pTank, pObject))
				return false;
		}
	}

	return true;
}

bool World::TestIntersect( const Object* pObj1, const Object* pObj2 )
{
	assert(pObj1 && pObj2);
	const hgeRect& rect1 = pObj1->GetBoundingBox();
	const hgeRect& rect2 = pObj2->GetBoundingBox();

	return TestIntersect(rect1, rect2);
}

bool World::TestIntersect( const hgeRect& rect1, const hgeRect& rect2 )
{
	hgeVector corners1[4] = { hgeVector(rect1.x1, rect1.y1), hgeVector(rect1.x1, rect1.y2)
		, hgeVector(rect1.x2, rect1.y1), hgeVector(rect1.x2, rect1.y2), };

	for (int i=0; i<4; ++i)
	{
		if (corners1[i].x >= rect2.x1 &&
			corners1[i].x <= rect2.x2 &&
			corners1[i].y >= rect2.y1 &&
			corners1[i].y <= rect2.y2	)
			return true;
	}

	return false;
}

void World::SetMainPlayer( Tank* player )
{
	m_player = player;

	if(m_player)
		m_player->SetIsEnemy(false);
}

ObjectVector World::GetBulletHitObject( const Bullet& bullet )
{
	ObjectVector retHitObjs;
	//Ƿеο
	hgeVector bulletPos = bullet.GetPos();
	float	bulletSize = bullet.GetSize();

	float tilePosX = bulletPos.x - fmod(bulletPos.x, TILE_SIZE);
	float tilePosY = bulletPos.y - fmod(bulletPos.y, TILE_SIZE);
	int tileIdxX = (int)(tilePosX / TILE_SIZE);
	int tileIdxY = (int)(tilePosY / TILE_SIZE);

	//ͬʱ2οŶ
	bool bFirstTile = false;
	Tile* pHitTile1 = GetTileLocation(tileIdxX, tileIdxY);
	if(pHitTile1)
	{
		retHitObjs.push_back(pHitTile1);
		bFirstTile = true;
	}

	eDirection bulletDir = bullet.GetDirection();
	if(bulletDir == eDirection_UP || bulletDir == eDirection_DOWN)
	{
		Tile* pHitTile2 = GetTileLocation(tileIdxX+1, tileIdxY);
		if(pHitTile2 && TestIntersect(&bullet, pHitTile2))
		{
			retHitObjs.push_back(pHitTile2);
			//ͬʱ2ο,򲻿ܻ̹,
			if(bFirstTile)
				return retHitObjs;
		}
	}
	else if(bulletDir == eDirection_LEFT || bulletDir == eDirection_RIGHT)	
	{
		Tile* pHitTile2 = GetTileLocation(tileIdxX, tileIdxY+1);
		if(pHitTile2 && TestIntersect(&bullet, pHitTile2))
		{
			retHitObjs.push_back(pHitTile2);
			//ͬʱ2ο,򲻿ܻ̹,
			if(bFirstTile)
				return retHitObjs;
		}
	}

	//hard wayӵÿ̹˵ཻ
	const ObjectList& objects = m_objectMgr.GetObjects();
	for (auto iter=objects.begin(); iter!=objects.end(); ++iter)
	{
		if ((*iter)->GetType() == eObjectType_Tank)
		{
			Tank* pTank = dynamic_cast<Tank*>(*iter);
			if(pTank->GetIsEnemy() == bullet.GetIsFromEnemy())
				continue;

			if(TestIntersect(&bullet, pTank))
				retHitObjs.push_back(pTank);
		}
	}

	return retHitObjs;
}

void World::AddToDestroyQueue( Object* pObj )
{
	//ȷΨһ
	if(std::find(m_vecTobeDestroy.begin(), m_vecTobeDestroy.end(), pObj) != m_vecTobeDestroy.end())
		return;

	m_vecTobeDestroy.push_back(pObj);
}

void World::_ProcessDestroyQueue()
{
	for (size_t i=0; i<m_vecTobeDestroy.size(); ++i)
	{
		//֪ͨԼݻ
		m_vecTobeDestroy[i]->OnDestroy();

		m_objectMgr.DestroyObject(m_vecTobeDestroy[i]);
	}
	m_vecTobeDestroy.clear();
}

void World::DestroyWorld()
{
	SAFE_DELETE(m_pFont);
	ResetWorld();
	m_objectMgr.UnloadRes();
}

bool World::InitWorld()
{
	//HGEԴ
	m_objectMgr.LoadRes();

	//йؿ
	if(!m_levelMgr.LoadAllLevels(m_strResPath+"Level\\"))
		return false;

	//ģ
	m_movieMgr.InitMovieTemplate(m_strResPath);

	//ʼHUD
	m_quadHUD.v[0].x = RESOLUTIONX;			m_quadHUD.v[0].y = 0;
	m_quadHUD.v[1].x = RESOLUTIONX+HUDX;	m_quadHUD.v[1].y = 0;
	m_quadHUD.v[2].x = RESOLUTIONX+HUDX;	m_quadHUD.v[2].y = RESOLUTIONY;
	m_quadHUD.v[3].x = RESOLUTIONX;			m_quadHUD.v[3].y = RESOLUTIONY;

	m_quadHUD.v[0].z = m_quadHUD.v[1].z = m_quadHUD.v[2].z = m_quadHUD.v[3].z = 0;
	m_quadHUD.v[0].col = m_quadHUD.v[1].col = m_quadHUD.v[2].col = m_quadHUD.v[3].col = 0xffdedede;
	m_quadHUD.tex = 0;

	m_pFont = new hgeFont("Media\\font1.fnt");
	m_pFont->SetColor(0xff000000);

	return true;
}

void World::RenderWorld()
{
	m_objectMgr.Render();
	m_movieMgr.Render();

	//ϳ
	float x = 15 * TILE_SIZE, y = (RESOLUTIONY / TILE_SIZE - 2) * TILE_SIZE;
	m_objectMgr.m_pFlag->RenderStretch(x, y, x+50, y+50);

	_RenderHUD();
}

void World::ResetWorld()
{
	if(m_objectMgr.m_pFlag)
		m_objectMgr.m_pFlag->SetTexture(m_objectMgr.m_texFlag);
	m_player = NULL;
	m_objectMgr.DestroyAllObjects();
	m_movieMgr.KillAllInstances();
	m_vecTileLoc.clear();
	m_vecTobeDestroy.clear();
	m_bornEnemyCnt = 0;
	m_bHitFlag = false;
	
}

void World::_RenderHUD()
{
	const int playerLife = m_levelMgr.GetPlayerLife();
	const int enemyCnt	= m_levelMgr.GetRemainingEnemyCount();
	const int row		= enemyCnt / 2;
	const int rem		= enemyCnt % 2;
	float yPos			= 100;

	m_objectMgr.m_pHGE->Gfx_RenderQuad(&m_quadHUD);
	m_pFont->printf(RESOLUTIONX+HUDX/2.0f, yPos, HGETEXT_CENTER, "Player Life = %d", playerLife);
	yPos+= 50;
	m_pFont->printf(RESOLUTIONX+HUDX/2.0f, yPos, HGETEXT_CENTER, "Enemy :");

	yPos += 50;
	for (int i=0; i<row; ++i)
	{
		m_objectMgr.m_pHUDTankSpr->Render(RESOLUTIONX+50, yPos);
		m_objectMgr.m_pHUDTankSpr->Render(RESOLUTIONX+100, yPos);

		yPos += 50;
	}

	if(rem)
		m_objectMgr.m_pHUDTankSpr->Render(RESOLUTIONX+50, yPos);
}

void World::BornNewEnemy( const hgeVector& bornPos )
{
	static int look = 0;
	HTEXTURE tex = look ? m_objectMgr.m_texEnemy1 : m_objectMgr.m_texEnemy2;
	hgeSprite* pSpr = new hgeSprite(tex, 0, 10, 50, 60);

	Object* pObj = m_objectMgr.CreateObject(pSpr, eObjectType_Tank);
	++m_bornEnemyCnt;

	pObj->SetPos(bornPos);
	Tank* pTank = dynamic_cast<Tank*>(pObj);
	pTank->SetIsEnemy(true);
	eDirection randDir = (eDirection)(rand() % eDirection_MaxCount);
	pTank->SetDirection(randDir);

	look = !look;
}

void World::BornPlayer()
{
	int vTileCnt = RESOLUTIONY / (int)TILE_SIZE;
	//ʼplayer̹½
	hgeSprite* pSpr = new hgeSprite(m_objectMgr.m_texTank, 0, 10, 50, 60);
	Object* player = m_objectMgr.CreateObject(pSpr, eObjectType_Tank);
	player->SetPos(hgeVector(0, (vTileCnt-2)*TILE_SIZE));
	SetMainPlayer((Tank*)player);
}

hgeVector World::GetRandomBornPos(bool bNoTile)
{
	int hTileCnt = RESOLUTIONX / (int)TILE_SIZE;
	int vTileCnt = RESOLUTIONY / (int)TILE_SIZE;
	int bornTileIdxX, bornTileIdxY;
	int curLoopCnt = 0, maxLoopCnt = hTileCnt * vTileCnt / 2;
	bool bHasTile = false;
	do 
	{
		bHasTile = false;

		bornTileIdxX = rand() % (hTileCnt - 1);
		bornTileIdxY = rand() % (vTileCnt / 2 - 1);

		//ʼղҲոӾͰ,ѭ
		if(++curLoopCnt >= maxLoopCnt || !bNoTile)
			break;

		//ռݵ4Ӷеο
		if(	GetTileLocation(bornTileIdxX, bornTileIdxY) ||
			GetTileLocation(bornTileIdxX+1, bornTileIdxY) ||
			GetTileLocation(bornTileIdxX, bornTileIdxY+1) ||
			GetTileLocation(bornTileIdxX+1, bornTileIdxY+1) )
			bHasTile = true;
	} while (bHasTile);


	return hgeVector(bornTileIdxX*TILE_SIZE, bornTileIdxY*TILE_SIZE);
}

void World::OnMovieEnd( HgeMovie* pMovie )
{
	BornNewEnemy(pMovie->Getpos());
}

void World::_AutoGenObject()
{
	static float pastTime = 0;
	pastTime += TIME_PER_FRAME;

	if(pastTime > ENEMY_BORN_INTERVAL && m_bornEnemyCnt < ENEMY_PER_LEVEL)
	{
		//ÿһʱһ
		HgeMovie* pBornEffect = m_movieMgr.PlayMovie("Born", GetRandomBornPos(true));
		pBornEffect->SetLoopInfo(2, true);
		pBornEffect->SetPauseTime(2);
		pBornEffect->AddListener(this);

		pastTime = 0;
	}

	//Ʒ
	static float itemGenTime = 0;
	itemGenTime += TIME_PER_FRAME;
	if (itemGenTime > ITEM_BORN_INTERVAL)
	{
		if(!GetCurrentItem())
		{
			eItem itemType = static_cast<eItem>(rand() % eItem_Count);
			hgeSprite* pSpr = new hgeSprite(m_objectMgr.m_vecItemTexs[itemType], 0, 0, (float)ITEM_SIZE, (float)ITEM_SIZE);

			Object* pObj = m_objectMgr.CreateObject(pSpr, eObjectType_Item);
			Item* pItem = dynamic_cast<Item*>(pObj);
			assert(pItem);
			pObj->SetPos(GetRandomBornPos(false));

			if(itemType == eItem_Grenade)
				pItem->SetFunc(std::tr1::bind(&World::EatItemGrenade, &World::GetSingleton()));
			else if(itemType == eItem_Star)
				pItem->SetFunc(std::tr1::bind(&World::EatItemStar, &World::GetSingleton()));

			m_objectMgr.m_pHGE->Effect_Play(m_objectMgr.m_itemBornSound);
		}

		itemGenTime = 0;
	}
}

Item* World::GetCurrentItem()
{
	const ObjectList& objects = World::GetSingleton().GetObjectManager().GetObjects();
	Item* pItem = nullptr;
	for (auto iter=objects.begin(); iter!=objects.end(); ++iter)
	{
		if((*iter)->GetType() == eObjectType_Item)
			pItem = dynamic_cast<Item*>(*iter);
	}

	return pItem;
}

void World::OnFlagBeenHit()
{
	m_objectMgr.m_pFlag->SetTexture(m_objectMgr.m_texFlagBroken);
	m_bHitFlag = true;
	SetGameOver(true);
}

void World::EatItemGrenade()
{
	const ObjectList& objects = m_objectMgr.GetObjects();
	for (auto iter=objects.begin(); iter!=objects.end(); ++iter)
	{
		Object* pObj = *iter;
		Tank* pTank = dynamic_cast<Tank*>(*iter);
		if (pObj->GetType() == eObjectType_Tank && pTank->GetIsEnemy())
			World::GetSingleton().AddToDestroyQueue(pObj);
	}

	m_objectMgr.m_pHGE->Effect_Play(m_objectMgr.m_bombSound);
}

void World::EatItemStar()
{
	m_levelMgr.AddPlayerLife();

	m_objectMgr.m_pHGE->Effect_Play(m_objectMgr.m_addLifeSound);
}
